@@ -42,6 +42,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <rtabmap/core/util3d_transforms.h>
 #include <rtabmap/core/Memory.h>
 #include <rtabmap/core/Signature.h>
+#include <rtabmap/core/Compression.h>
 #include "rtabmap_conversions/MsgConversion.h"
 #include "rtabmap_msgs/OdomInfo.h"
 #include "rtabmap/utilite/UConversion.h"
@@ -58,8 +59,6 @@ namespace rtabmap_odom {
 
 OdometryROS::OdometryROS(bool stereoParams, bool visParams, bool icpParams) :
 	odometry_(0),
-	warningThread_(0),
-	callbackCalled_(false),
 	frameId_("base_link"),
 	odomFrameId_("odom"),
 	groundTruthFrameId_(""),
@@ -72,6 +71,7 @@ OdometryROS::OdometryROS(bool stereoParams, bool visParams, bool icpParams) :
 	waitForTransform_(true),
 	waitForTransformDuration_(0.1), // 100 ms
 	publishNullWhenLost_(true),
+	publishCompressedSensorData_(false),
 	paused_(false),
 	resetCountdown_(0),
 	resetCurrentCount_(0),
@@ -82,6 +82,8 @@ OdometryROS::OdometryROS(bool stereoParams, bool visParams, bool icpParams) :
 	expectedUpdateRate_(0.0),
 	maxUpdateRate_(0.0),
 	minUpdateRate_(0.0),
+	compressionImgFormat_(".jpg"),
+	compressionParallelized_(true),
 	odomStrategy_(Parameters::defaultOdomStrategy()),
 	waitIMUToinit_(false),
 	imuProcessed_(false)
@@ -91,13 +93,7 @@ OdometryROS::OdometryROS(bool stereoParams, bool visParams, bool icpParams) :
 
 OdometryROS::~OdometryROS()
 {
-	if(warningThread_)
-	{
-		callbackCalled();
-		warningThread_->join();
-		delete warningThread_;
-	}
-
+	this->join(true);
 	delete odometry_;
 }
 
@@ -113,6 +109,9 @@ void OdometryROS::onInit()
 	odomLocalScanMap_ = nh.advertise<sensor_msgs::PointCloud2>("odom_local_scan_map", 1);
 	odomLastFrame_ = nh.advertise<sensor_msgs::PointCloud2>("odom_last_frame", 1);
 	odomRgbdImagePub_ = nh.advertise<rtabmap_msgs::RGBDImage>("odom_rgbd_image", 1);
+	odomSensorDataPub_ = nh.advertise<rtabmap_msgs::SensorData>("odom_sensor_data/raw", 1);
+	odomSensorDataFeaturesPub_ = nh.advertise<rtabmap_msgs::SensorData>("odom_sensor_data/features", 1);
+	odomSensorDataCompressedPub_ = nh.advertise<rtabmap_msgs::SensorData>("odom_sensor_data/compressed", 1);
 
 	Transform initialPose = Transform::getIdentity();
 	std::string initialPoseStr;
@@ -151,6 +150,9 @@ void OdometryROS::onInit()
 	pnh.param("max_update_rate", maxUpdateRate_, maxUpdateRate_);
 	pnh.param("min_update_rate", minUpdateRate_, minUpdateRate_);
 
+	pnh.param("sensor_data_compression_format", compressionImgFormat_, compressionImgFormat_);
+	pnh.param("sensor_data_parallel_compression", compressionParallelized_, compressionParallelized_);
+
 	pnh.param("wait_imu_to_init", waitIMUToinit_, waitIMUToinit_);
 
 	int eventLevel = ULogger::kFatal;
@@ -176,6 +178,7 @@ void OdometryROS::onInit()
 	NODELET_INFO("Odometry: ground_truth_base_frame_id = %s", groundTruthBaseFrameId_.c_str());
 	NODELET_INFO("Odometry: config_path            = %s", configPath.c_str());
 	NODELET_INFO("Odometry: publish_null_when_lost = %s", publishNullWhenLost_?"true":"false");
+	NODELET_INFO("Odometry: publish_compressed_sensor_data = %s", publishCompressedSensorData_?"true":"false");
 	NODELET_INFO("Odometry: guess_frame_id         = %s", guessFrameId_.c_str());
 	NODELET_INFO("Odometry: guess_min_translation  = %f", guessMinTranslation_);
 	NODELET_INFO("Odometry: guess_min_rotation     = %f", guessMinRotation_);
@@ -184,6 +187,8 @@ void OdometryROS::onInit()
 	NODELET_INFO("Odometry: max_update_rate        = %f Hz", maxUpdateRate_);
 	NODELET_INFO("Odometry: min_update_rate        = %f Hz", minUpdateRate_);
 	NODELET_INFO("Odometry: wait_imu_to_init       = %s", waitIMUToinit_?"true":"false");
+	NODELET_INFO("Odometry: sensor_data_compression_format   = %s", compressionImgFormat_.c_str());
+	NODELET_INFO("Odometry: sensor_data_parallel_compression = %s", compressionParallelized_?"true":"false");
 
 	configPath = uReplaceChar(configPath, '~', UDirectory::homeDir());
 	if(configPath.size() && configPath.at(0) != '/')
@@ -375,32 +380,26 @@ void OdometryROS::onInit()
 		NODELET_INFO("odometry: Subscribing to IMU topic %s", imuSub_.getTopic().c_str());
 	}
 
+	this->start();
+
 	onOdomInit();
 }
 
-void OdometryROS::startWarningThread(const std::string & subscribedTopicsMsg, bool approxSync)
+void OdometryROS::initDiagnosticMsg(const std::string & subscribedTopicsMsg, bool approxSync, const std::string & subscribedTopic)
 {
-	warningThread_ = new boost::thread(boost::bind(&OdometryROS::warningLoop, this, subscribedTopicsMsg, approxSync));
 	NODELET_INFO("%s", subscribedTopicsMsg.c_str());
-}
-
-void OdometryROS::warningLoop(const std::string & subscribedTopicsMsg, bool approxSync)
-{
-	ros::Duration r(5.0);
-	while(!callbackCalled_)
-	{
-		r.sleep();
-		if(!callbackCalled_)
-		{
-			ROS_WARN("%s: Did not receive data since 5 seconds! Make sure the input topics are "
+	syncDiagnostic_.reset(new rtabmap_sync::SyncDiagnostic(getNodeHandle(), getPrivateNodeHandle(), getName(), 0.5));
+	std::vector<diagnostic_updater::DiagnosticTask*> tasks;
+	tasks.push_back(&statusDiagnostic_);
+	syncDiagnostic_->init(subscribedTopic,
+		uFormat("%s: Did not receive data since 5 seconds! Make sure the input topics are "
 					"published (\"$ rostopic hz my_topic\") and the timestamps in their "
 					"header are set. %s%s",
 					getName().c_str(),
 					approxSync?"":"Parameter \"approx_sync\" is false, which means that input "
 						"topics should have all the exact timestamp for the callback to be called.",
-					subscribedTopicsMsg.c_str());
-		}
-	}
+					subscribedTopicsMsg.c_str()),
+		tasks);
 }
 
 rtabmap::Transform OdometryROS::velocityGuess() const
@@ -437,58 +436,105 @@ void OdometryROS::callbackIMU(const sensor_msgs::ImuConstPtr& msg)
 				cv::Mat(3,3,CV_64FC1,(void*)msg->linear_acceleration_covariance.data()).clone(),
 				localTransform);
 
-		imus_.insert(std::make_pair(stamp, imu));
-
-		if(bufferedData_.first.isValid() && stamp > bufferedData_.first.stamp())
 		{
-			SensorData data = bufferedData_.first;
-			bufferedData_.first = SensorData();
-			processData(data, bufferedData_.second);
+			UScopeMutex m(imuMutex_);
+
+			imus_.insert(std::make_pair(stamp, imu));
+			if(imus_.size() > 1000)
+			{
+				NODELET_WARN("Dropping imu data!");
+				imus_.erase(imus_.begin());
+			}
 		}
-
-		if(imus_.size() > 1000)
+		if(dataMutex_.lockTry() == 0)
 		{
-			imus_.erase(imus_.begin());
+			if(bufferedDataToProcess_ && dataHeaderToProcess_.stamp.toSec() <= stamp)
+			{
+				bufferedDataToProcess_ = false;
+				dataReady_.release();
+			}
+			dataMutex_.unlock();
 		}
 	}
 }
 
 void OdometryROS::processData(SensorData & data, const std_msgs::Header & header)
 {
-	if((waitIMUToinit_ && !imuProcessed_) && odometry_->framesProcessed() == 0 && odometry_->getPose().isIdentity() && imus_.empty())
+	//NODELET_WARN("Received image: %f delay=%f", data.stamp(), (ros::Time::now() - header.stamp).toSec());
+	if(dataMutex_.lockTry() == 0)
 	{
-		NODELET_WARN("odometry: waiting imu (%s) to initialize orientation (wait_imu_to_init=true)", imuSub_.getTopic().c_str());
-		return;
-	}
-
-	if(waitIMUToinit_ && (imus_.empty() || imus_.rbegin()->first < header.stamp.toSec()))
-	{
-		//NODELET_WARN("No imu received with higher stamp than last image (%f)! Buffering this image until we get more imu msgs...", stamp.toSec());
-
-		// keep in cache to process later when we will receive imu msgs
-		if(bufferedData_.first.isValid())
-		{
-			NODELET_ERROR("Overwriting previous data! Make sure IMU is "
-					"published faster than data rate. (last image stamp "
-					"buffered=%f and new one is %f, last imu stamp received=%f)",
-					bufferedData_.first.stamp(), data.stamp(), imus_.empty()?0:imus_.rbegin()->first);
+		if(bufferedDataToProcess_) {
+			NODELET_ERROR("We didn't receive IMU newer than previous image (%f) and we just received a new image (%f). The previous image is dropped!",
+						dataHeaderToProcess_.stamp.toSec(), header.stamp.toSec());
 		}
-		bufferedData_.first = data;
-		bufferedData_.second = header;
+		dataToProcess_ = data;
+		dataHeaderToProcess_ = header;
+		bufferedDataToProcess_ = false;
+		dataReady_.release();
+		dataMutex_.unlock();
+	}
+	else
+	{
+		NODELET_DEBUG("Dropping image/scan data");
+	}
+}
+
+void OdometryROS::mainLoopKill()
+{
+	// in case we were waiting, unblock thread
+	dataReady_.release();
+}
+
+void OdometryROS::mainLoop()
+{
+	dataReady_.acquire();
+
+	if(!this->isRunning())
+	{
+		// thread killed
 		return;
 	}
-	// process all imu data up to current image stamp (or just after so that underlying odom approach can do interpolation of imu at image stamp)
-	std::map<double, rtabmap::IMU>::iterator iterEnd = imus_.lower_bound(header.stamp.toSec());
-	if(iterEnd!= imus_.end())
+
+	UScopeMutex lock(dataMutex_);
+
+	// aliases
+	SensorData & data = dataToProcess_;
+	std_msgs::Header & header = dataHeaderToProcess_;
+
+	std::vector<std::pair<double, IMU> > imus;
 	{
-		++iterEnd;
+		UScopeMutex m(imuMutex_);
+
+		if((waitIMUToinit_ && !imuProcessed_) && odometry_->framesProcessed() == 0 && odometry_->getPose().isIdentity() && imus_.empty())
+		{
+			NODELET_WARN("odometry: waiting imu (%s) to initialize orientation (wait_imu_to_init=true)", imuSub_.getTopic().c_str());
+			return;
+		}
+
+		if(waitIMUToinit_ && (imus_.empty() || imus_.rbegin()->first < header.stamp.toSec()))
+		{
+			NODELET_WARN("Make sure IMU is published faster than data rate! (last image stamp=%f and last imu stamp received=%f). Buffering the image until an imu with same or greater stamp is received.",
+					data.stamp(), imus_.empty()?0:imus_.rbegin()->first);
+			bufferedDataToProcess_ = true;
+			return;
+		}
+		// process all imu data up to current image stamp (or just after so that underlying odom approach can do interpolation of imu at image stamp)
+		std::map<double, rtabmap::IMU>::iterator iterEnd = imus_.lower_bound(header.stamp.toSec());
+		if(iterEnd!= imus_.end())
+		{
+			++iterEnd;
+		}
+		for(std::map<double, rtabmap::IMU>::iterator iter=imus_.begin(); iter!=iterEnd;)
+		{
+			imus.push_back(*iter);
+			imus_.erase(iter++);
+		}
 	}
-	for(std::map<double, rtabmap::IMU>::iterator iter=imus_.begin(); iter!=iterEnd;)
+
+	for(size_t i=0; i<imus.size(); ++i)
 	{
-		//NODELET_WARN("img callback: process imu   %f", iter->first);
-		SensorData dataIMU(iter->second, 0, iter->first);
+		SensorData dataIMU(imus[i].second, 0, imus[i].first);
 		odometry_->process(dataIMU);
-		imus_.erase(iter++);
 		imuProcessed_ = true;
 	}
 
@@ -859,10 +905,9 @@ void OdometryROS::processData(SensorData & data, const std_msgs::Header & header
 					"is %fs too old (>%fs, min_update_rate = %f Hz). Previous data stamp is %f while new data stamp is %f.",
 					header.stamp.toSec() - previousStamp_, 1.0/minUpdateRate_, minUpdateRate_, previousStamp_, header.stamp.toSec());
 		}
-		else
+		else if(--resetCurrentCount_>0)
 		{
 			NODELET_WARN( "Odometry lost! Odometry will be reset after next %d consecutive unsuccessful odometry updates...", resetCurrentCount_);
-			--resetCurrentCount_;
 		}
 
 		if(resetCurrentCount_ == 0 || tooOldPreviousData)
@@ -890,6 +935,11 @@ void OdometryROS::processData(SensorData & data, const std_msgs::Header & header
 					odometry_->reset(tfPose);
 				}
 			}
+			// Keep resetting if the odometry cannot initialize in next updates (e.g., lack of features).
+			// This will make sure we keep updating to latest guess pose.
+			if(resetCurrentCount_ == 0) {
+				++resetCurrentCount_;
+			}
 		}
 	}
 
@@ -919,6 +969,8 @@ void OdometryROS::processData(SensorData & data, const std_msgs::Header & header
 		}
 	}
 
+	postProcessData(data, header);
+
 	if(!data.imageRaw().empty() && odomRgbdImagePub_.getNumSubscribers())
 	{
 		if(!header.frame_id.empty())
@@ -934,27 +986,122 @@ void OdometryROS::processData(SensorData & data, const std_msgs::Header & header
 		}
 	}
 
-	postProcessData(data, header);
-
-	if(!data.imageRaw().empty() || !data.laserScanRaw().isEmpty())
+	if(odomSensorDataPub_.getNumSubscribers() || odomSensorDataFeaturesPub_.getNumSubscribers())
 	{
-		if(visParams_)
+		rtabmap_msgs::SensorData msg;
+		rtabmap_conversions::sensorDataToROS(data, msg, frameId_, odomSensorDataPub_.getNumSubscribers());
+		msg.header.stamp = header.stamp; // use corresponding time stamp to image
+		if(odomSensorDataPub_.getNumSubscribers())
 		{
-			if(icpParams_)
-			{
-				NODELET_INFO( "Odom: quality=%d, ratio=%f, std dev=%fm|%frad, update time=%fs", info.reg.inliers, info.reg.icpInliersRatio, pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(0,0)), pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(5,5)), (ros::WallTime::now()-time).toSec());
-			}
-			else
-			{
-				NODELET_INFO( "Odom: quality=%d, std dev=%fm|%frad, update time=%fs", info.reg.inliers, pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(0,0)), pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(5,5)), (ros::WallTime::now()-time).toSec());
-			}
+			odomSensorDataPub_.publish(msg);
 		}
-		else // if(icpParams_)
+		if(odomSensorDataFeaturesPub_.getNumSubscribers())
 		{
-			NODELET_INFO( "Odom: ratio=%f, std dev=%fm|%frad, update time=%fs", info.reg.icpInliersRatio, pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(0,0)), pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(5,5)), (ros::WallTime::now()-time).toSec());
+			// remove data
+			msg.left = sensor_msgs::Image();
+			msg.right = sensor_msgs::Image();
+			msg.laser_scan = sensor_msgs::PointCloud2();
+			msg.grid_ground.clear();
+			msg.grid_obstacles.clear();
+			msg.grid_empty_cells.clear();
+			odomSensorDataFeaturesPub_.publish(msg);
 		}
-		previousStamp_ = header.stamp.toSec();
 	}
+	if(odomSensorDataCompressedPub_.getNumSubscribers())
+	{
+		cv::Mat compressedImage;
+		cv::Mat compressedDepth;
+		cv::Mat compressedScan;
+		if(compressionParallelized_)
+		{
+			rtabmap::CompressionThread ctImage(data.imageRaw(), compressionImgFormat_);
+			rtabmap::CompressionThread ctDepth(data.depthOrRightRaw(), data.depthOrRightRaw().type() == CV_32FC1 || data.depthOrRightRaw().type() == CV_16UC1?std::string(".png"):compressionImgFormat_);
+			rtabmap::CompressionThread ctLaserScan(data.laserScanRaw().data());
+			if(!data.imageRaw().empty())
+			{
+				ctImage.start();
+			}
+			if(!data.depthOrRightRaw().empty())
+			{
+				ctDepth.start();
+			}
+			if(!data.laserScanRaw().isEmpty())
+			{
+				ctLaserScan.start();
+			}
+			ctImage.join();
+			ctDepth.join();
+			ctLaserScan.join();
+
+			compressedImage = ctImage.getCompressedData();
+			compressedDepth = ctDepth.getCompressedData();
+			compressedScan = ctLaserScan.getCompressedData();
+		}
+		else
+		{
+			compressedImage = compressImage2(data.imageRaw(), compressionImgFormat_);
+			compressedDepth = compressImage2(data.depthOrRightRaw(), data.depthOrRightRaw().type() == CV_32FC1 || data.depthOrRightRaw().type() == CV_16UC1?std::string(".png"):compressionImgFormat_);
+			compressedScan = compressData2(data.laserScanRaw().data());
+		}
+		if(!compressedImage.empty() && !data.stereoCameraModels().empty())
+		{
+			data.setStereoImage(compressedImage, compressedDepth, data.stereoCameraModels(), false);
+		}
+		else if(!compressedImage.empty() && !data.cameraModels().empty())
+		{
+			data.setRGBDImage(compressedImage, compressedDepth, data.cameraModels(), false);
+		}
+		if(!compressedScan.empty())
+		{
+			data.setLaserScan(data.laserScanRaw().angleIncrement() == 0.0f?
+						LaserScan(compressedScan,
+							data.laserScanRaw().maxPoints(),
+							data.laserScanRaw().rangeMax(),
+							data.laserScanRaw().format(),
+							data.laserScanRaw().localTransform()):
+						LaserScan(compressedScan,
+							data.laserScanRaw().format(),
+							data.laserScanRaw().rangeMin(),
+							data.laserScanRaw().rangeMax(),
+							data.laserScanRaw().angleMin(),
+							data.laserScanRaw().angleMax(),
+							data.laserScanRaw().angleIncrement(),
+							data.laserScanRaw().localTransform()), false);
+		}
+		rtabmap_msgs::SensorData msg;
+		rtabmap_conversions::sensorDataToROS(data, msg, frameId_, false);
+		msg.header.stamp = header.stamp; // use corresponding time stamp to image
+		odomSensorDataCompressedPub_.publish(msg);
+	}
+
+	double delay = (ros::Time::now() - header.stamp).toSec(); 
+	if(visParams_)
+	{
+		if(icpParams_)
+		{
+			NODELET_INFO( "Odom: quality=%d, ratio=%f, std dev=%fm|%frad, update time=%fs, delay=%fs", info.reg.inliers, info.reg.icpInliersRatio, pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(0,0)), pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(5,5)), (ros::WallTime::now()-time).toSec(), delay);
+		}
+		else
+		{
+			NODELET_INFO( "Odom: quality=%d, std dev=%fm|%frad, update time=%fs, delay=%fs", info.reg.inliers, pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(0,0)), pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(5,5)), (ros::WallTime::now()-time).toSec(), delay);
+		}
+	}
+	else // if(icpParams_)
+	{
+		NODELET_INFO( "Odom: ratio=%f, std dev=%fm|%frad, update time=%fs, delay=%fs", info.reg.icpInliersRatio, pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(0,0)), pose.isNull()?0.0f:std::sqrt(info.reg.covariance.at<double>(5,5)), (ros::WallTime::now()-time).toSec(), delay);
+	}
+
+	statusDiagnostic_.setStatus(pose.isNull());
+	if(syncDiagnostic_.get())
+	{
+		double curentRate = 1.0/(ros::WallTime::now()-time).toSec();
+		syncDiagnostic_->tick(header.stamp,
+			maxUpdateRate_>0 ? maxUpdateRate_:
+			expectedUpdateRate_>0 && expectedUpdateRate_ < curentRate ? expectedUpdateRate_:
+			previousStamp_ == 0.0 || header.stamp.toSec() - previousStamp_ > 1.0/curentRate?0:curentRate);
+	}
+
+	previousStamp_ = header.stamp.toSec();
 }
 
 bool OdometryROS::reset(std_srvs::Empty::Request&, std_srvs::Empty::Response&)
@@ -974,14 +1121,19 @@ bool OdometryROS::resetToPose(rtabmap_msgs::ResetPose::Request& req, rtabmap_msg
 
 void OdometryROS::reset(const Transform & pose)
 {
+	UScopeMutex lock(dataMutex_);
 	odometry_->reset(pose);
 	guess_.setNull();
 	guessPreviousPose_.setNull();
 	previousStamp_ = 0.0;
 	resetCurrentCount_ = resetCountdown_;
 	imuProcessed_ = false;
-	bufferedData_.first= SensorData();
+	dataToProcess_ = SensorData();
+	dataHeaderToProcess_ = std_msgs::Header();
+	bufferedDataToProcess_ = false;
+	imuMutex_.lock();
 	imus_.clear();
+	imuMutex_.unlock();
 	this->flushCallbacks();
 }
 
@@ -1038,5 +1190,32 @@ bool OdometryROS::setLogError(std_srvs::Empty::Request&, std_srvs::Empty::Respon
 	return true;
 }
 
+OdometryROS::OdomStatusTask::OdomStatusTask() :
+		diagnostic_updater::DiagnosticTask("Odom status"),
+		lost_(false),
+		dataReceived_(false)
+{}
+
+void OdometryROS::OdomStatusTask::setStatus(bool isLost)
+{
+	dataReceived_ = true;
+	lost_ = isLost;
+}
+
+void OdometryROS::OdomStatusTask::run(diagnostic_updater::DiagnosticStatusWrapper &stat)
+{
+	if(!dataReceived_)
+	{
+		stat.summary(diagnostic_msgs::DiagnosticStatus::ERROR, "No data received!");
+	}
+	else if(lost_)
+	{
+		stat.summary(diagnostic_msgs::DiagnosticStatus::ERROR, "Lost!");
+	}
+	else
+	{
+		stat.summary(diagnostic_msgs::DiagnosticStatus::OK, "Tracking.");
+	}
+}
 
 }
