@@ -72,7 +72,10 @@ public:
 		exactSync4_(0),
 		approxSync5_(0),
 		exactSync5_(0),
-		queueSize_(5),
+		approxSync6_(0),
+		exactSync6_(0),
+		topicQueueSize_(1),
+		syncQueueSize_(5),
 		keepColor_(false)
 	{
 	}
@@ -81,46 +84,18 @@ public:
 	{
 		rgbdSub_.shutdown();
 		rgbdxSub_.shutdown();
-		if(approxSync_)
-		{
-			delete approxSync_;
-		}
-		if(exactSync_)
-		{
-			delete exactSync_;
-		}
-		if(approxSync2_)
-		{
-			delete approxSync2_;
-		}
-		if(exactSync2_)
-		{
-			delete exactSync2_;
-		}
-		if(approxSync3_)
-		{
-			delete approxSync3_;
-		}
-		if(exactSync3_)
-		{
-			delete exactSync3_;
-		}
-		if(approxSync4_)
-		{
-			delete approxSync4_;
-		}
-		if(exactSync4_)
-		{
-			delete exactSync4_;
-		}
-		if(approxSync5_)
-		{
-			delete approxSync5_;
-		}
-		if(exactSync5_)
-		{
-			delete exactSync5_;
-		}
+		delete approxSync_;
+		delete exactSync_;
+		delete approxSync2_;
+		delete exactSync2_;
+		delete approxSync3_;
+		delete exactSync3_;
+		delete approxSync4_;
+		delete exactSync4_;
+		delete approxSync5_;
+		delete exactSync5_;
+		delete approxSync6_;
+		delete exactSync6_;
 	}
 
 private:
@@ -136,7 +111,19 @@ private:
 		double approxSyncMaxInterval = 0.0;
 		pnh.param("approx_sync", approxSync, approxSync);
 		pnh.param("approx_sync_max_interval", approxSyncMaxInterval, approxSyncMaxInterval);
-		pnh.param("queue_size", queueSize_, queueSize_);
+		pnh.param("topic_queue_size", topicQueueSize_, topicQueueSize_);
+		if(pnh.hasParam("queue_size") && !pnh.hasParam("sync_queue_size"))
+		{
+			pnh.param("queue_size", syncQueueSize_, syncQueueSize_);
+			ROS_WARN("Parameter \"queue_size\" has been renamed "
+					"to \"sync_queue_size\" and will be removed "
+					"in future versions! The value (%d) is still copied to "
+					"\"sync_queue_size\".", syncQueueSize_);
+		}
+		else
+		{
+			pnh.param("sync_queue_size", syncQueueSize_, syncQueueSize_);
+		}
 		pnh.param("subscribe_rgbd", subscribeRGBD, subscribeRGBD);
 		if(pnh.hasParam("depth_cameras"))
 		{
@@ -147,38 +134,40 @@ private:
 		{
 			rgbdCameras = 0;
 		}
-		if(rgbdCameras > 5)
-		{
-			NODELET_FATAL("Only 5 cameras maximum supported yet. Set 0 to use rgbd_images input (for which rgbdx_sync node can sync up to 8 cameras).");
-		}
 		pnh.param("keep_color", keepColor_, keepColor_);
 
 		NODELET_INFO("RGBDOdometry: approx_sync    = %s", approxSync?"true":"false");
 		if(approxSync)
 			NODELET_INFO("RGBDOdometry: approx_sync_max_interval = %f", approxSyncMaxInterval);
-		NODELET_INFO("RGBDOdometry: queue_size     = %d", queueSize_);
+		NODELET_INFO("RGBDOdometry: topic_queue_size = %d", topicQueueSize_);
+		NODELET_INFO("RGBDOdometry: sync_queue_size  = %d", syncQueueSize_);
 		NODELET_INFO("RGBDOdometry: subscribe_rgbd = %s", subscribeRGBD?"true":"false");
 		NODELET_INFO("RGBDOdometry: rgbd_cameras   = %d", rgbdCameras);
 		NODELET_INFO("RGBDOdometry: keep_color     = %s", keepColor_?"true":"false");
 
+		std::string subscribedTopic;
 		std::string subscribedTopicsMsg;
 		if(subscribeRGBD)
 		{
 			if(rgbdCameras >= 2)
 			{
-				rgbd_image1_sub_.subscribe(nh, "rgbd_image0", 1);
-				rgbd_image2_sub_.subscribe(nh, "rgbd_image1", 1);
+				rgbd_image1_sub_.subscribe(nh, "rgbd_image0", topicQueueSize_);
+				rgbd_image2_sub_.subscribe(nh, "rgbd_image1", topicQueueSize_);
 				if(rgbdCameras >= 3)
 				{
-					rgbd_image3_sub_.subscribe(nh, "rgbd_image2", 1);
+					rgbd_image3_sub_.subscribe(nh, "rgbd_image2", topicQueueSize_);
 				}
 				if(rgbdCameras >= 4)
 				{
-					rgbd_image4_sub_.subscribe(nh, "rgbd_image3", 1);
+					rgbd_image4_sub_.subscribe(nh, "rgbd_image3", topicQueueSize_);
 				}
 				if(rgbdCameras >= 5)
 				{
-					rgbd_image5_sub_.subscribe(nh, "rgbd_image4", 1);
+					rgbd_image5_sub_.subscribe(nh, "rgbd_image4", topicQueueSize_);
+				}
+				if(rgbdCameras >= 6)
+				{
+					rgbd_image6_sub_.subscribe(nh, "rgbd_image5", topicQueueSize_);
 				}
 
 				if(rgbdCameras == 2)
@@ -186,7 +175,7 @@ private:
 					if(approxSync)
 					{
 						approxSync2_ = new message_filters::Synchronizer<MyApproxSync2Policy>(
-								MyApproxSync2Policy(queueSize_),
+								MyApproxSync2Policy(syncQueueSize_),
 								rgbd_image1_sub_,
 								rgbd_image2_sub_);
 						if(approxSyncMaxInterval > 0.0)
@@ -196,7 +185,7 @@ private:
 					else
 					{
 						exactSync2_ = new message_filters::Synchronizer<MyExactSync2Policy>(
-								MyExactSync2Policy(queueSize_),
+								MyExactSync2Policy(syncQueueSize_),
 								rgbd_image1_sub_,
 								rgbd_image2_sub_);
 						exactSync2_->registerCallback(boost::bind(&RGBDOdometry::callbackRGBD2, this, boost::placeholders::_1, boost::placeholders::_2));
@@ -213,7 +202,7 @@ private:
 					if(approxSync)
 					{
 						approxSync3_ = new message_filters::Synchronizer<MyApproxSync3Policy>(
-								MyApproxSync3Policy(queueSize_),
+								MyApproxSync3Policy(syncQueueSize_),
 								rgbd_image1_sub_,
 								rgbd_image2_sub_,
 								rgbd_image3_sub_);
@@ -224,7 +213,7 @@ private:
 					else
 					{
 						exactSync3_ = new message_filters::Synchronizer<MyExactSync3Policy>(
-								MyExactSync3Policy(queueSize_),
+								MyExactSync3Policy(syncQueueSize_),
 								rgbd_image1_sub_,
 								rgbd_image2_sub_,
 								rgbd_image3_sub_);
@@ -243,7 +232,7 @@ private:
 					if(approxSync)
 					{
 						approxSync4_ = new message_filters::Synchronizer<MyApproxSync4Policy>(
-								MyApproxSync4Policy(queueSize_),
+								MyApproxSync4Policy(syncQueueSize_),
 								rgbd_image1_sub_,
 								rgbd_image2_sub_,
 								rgbd_image3_sub_,
@@ -255,7 +244,7 @@ private:
 					else
 					{
 						exactSync4_ = new message_filters::Synchronizer<MyExactSync4Policy>(
-								MyExactSync4Policy(queueSize_),
+								MyExactSync4Policy(syncQueueSize_),
 								rgbd_image1_sub_,
 								rgbd_image2_sub_,
 								rgbd_image3_sub_,
@@ -276,7 +265,7 @@ private:
 					if(approxSync)
 					{
 						approxSync5_ = new message_filters::Synchronizer<MyApproxSync5Policy>(
-								MyApproxSync5Policy(queueSize_),
+								MyApproxSync5Policy(syncQueueSize_),
 								rgbd_image1_sub_,
 								rgbd_image2_sub_,
 								rgbd_image3_sub_,
@@ -289,7 +278,7 @@ private:
 					else
 					{
 						exactSync5_ = new message_filters::Synchronizer<MyExactSync5Policy>(
-								MyExactSync5Policy(queueSize_),
+								MyExactSync5Policy(syncQueueSize_),
 								rgbd_image1_sub_,
 								rgbd_image2_sub_,
 								rgbd_image3_sub_,
@@ -307,11 +296,58 @@ private:
 							rgbd_image4_sub_.getTopic().c_str(),
                                                         rgbd_image5_sub_.getTopic().c_str());
 				}
+				else if(rgbdCameras == 6)
+				{
+					if(approxSync)
+					{
+						approxSync6_ = new message_filters::Synchronizer<MyApproxSync6Policy>(
+								MyApproxSync6Policy(syncQueueSize_),
+								rgbd_image1_sub_,
+								rgbd_image2_sub_,
+								rgbd_image3_sub_,
+								rgbd_image4_sub_,
+								rgbd_image5_sub_,
+								rgbd_image6_sub_);
+						if(approxSyncMaxInterval > 0.0)
+							approxSync6_->setMaxIntervalDuration(ros::Duration(approxSyncMaxInterval));
+						approxSync6_->registerCallback(boost::bind(&RGBDOdometry::callbackRGBD6, this, boost::placeholders::_1, boost::placeholders::_2, boost::placeholders::_3, boost::placeholders::_4, boost::placeholders::_5, boost::placeholders::_6));
+					}
+					else
+					{
+						exactSync6_ = new message_filters::Synchronizer<MyExactSync6Policy>(
+								MyExactSync6Policy(syncQueueSize_),
+								rgbd_image1_sub_,
+								rgbd_image2_sub_,
+								rgbd_image3_sub_,
+								rgbd_image4_sub_,
+								rgbd_image5_sub_,
+								rgbd_image6_sub_);
+						exactSync6_->registerCallback(boost::bind(&RGBDOdometry::callbackRGBD6, this, boost::placeholders::_1, boost::placeholders::_2, boost::placeholders::_3, boost::placeholders::_4, boost::placeholders::_5, boost::placeholders::_6));
+					}
+					subscribedTopicsMsg = uFormat("\n%s subscribed to (%s sync%s):\n   %s \\\n  %s \\\n  %s \\\n   %s \\\n   %s \\\n   %s",
+							getName().c_str(),
+							approxSync?"approx":"exact",
+							approxSync&&approxSyncMaxInterval!=0.0?uFormat(", max interval=%fs", approxSyncMaxInterval).c_str():"",
+							rgbd_image1_sub_.getTopic().c_str(),
+							rgbd_image2_sub_.getTopic().c_str(),
+							rgbd_image3_sub_.getTopic().c_str(),
+							rgbd_image4_sub_.getTopic().c_str(),
+							rgbd_image5_sub_.getTopic().c_str(),
+							rgbd_image6_sub_.getTopic().c_str());
+				}
+				else
+				{
+					NODELET_FATAL("%s doesn't support more than 6 cameras (rgbd_cameras=%d) with "
+								  "internal synchronization interface, set rgbd_cameras=0 and use "
+								  "rgbd_images input topic instead for more cameras (for which "
+								  "rgbdx_sync node can sync up to 8 cameras).",
+								  getName().c_str(), rgbdCameras);
+				}
 
 			}
 			else if(rgbdCameras == 0)
 			{
-				rgbdxSub_ = nh.subscribe("rgbd_images", 1, &RGBDOdometry::callbackRGBDX, this);
+				rgbdxSub_ = nh.subscribe("rgbd_images", topicQueueSize_, &RGBDOdometry::callbackRGBDX, this);
 
 				subscribedTopicsMsg =
 						uFormat("\n%s subscribed to:\n   %s",
@@ -320,7 +356,7 @@ private:
 			}
 			else
 			{
-				rgbdSub_ = nh.subscribe("rgbd_image", 1, &RGBDOdometry::callbackRGBD, this);
+				rgbdSub_ = nh.subscribe("rgbd_image", topicQueueSize_, &RGBDOdometry::callbackRGBD, this);
 
 				subscribedTopicsMsg =
 						uFormat("\n%s subscribed to:\n   %s",
@@ -339,23 +375,24 @@ private:
 			image_transport::TransportHints hintsRgb("raw", ros::TransportHints(), rgb_pnh);
 			image_transport::TransportHints hintsDepth("raw", ros::TransportHints(), depth_pnh);
 
-			image_mono_sub_.subscribe(rgb_it, rgb_nh.resolveName("image"), 1, hintsRgb);
-			image_depth_sub_.subscribe(depth_it, depth_nh.resolveName("image"), 1, hintsDepth);
-			info_sub_.subscribe(rgb_nh, "camera_info", 1);
+			image_mono_sub_.subscribe(rgb_it, rgb_nh.resolveName("image"), topicQueueSize_, hintsRgb);
+			image_depth_sub_.subscribe(depth_it, depth_nh.resolveName("image"), topicQueueSize_, hintsDepth);
+			info_sub_.subscribe(rgb_nh, "camera_info", topicQueueSize_);
 
 			if(approxSync)
 			{
-				approxSync_ = new message_filters::Synchronizer<MyApproxSyncPolicy>(MyApproxSyncPolicy(queueSize_), image_mono_sub_, image_depth_sub_, info_sub_);
+				approxSync_ = new message_filters::Synchronizer<MyApproxSyncPolicy>(MyApproxSyncPolicy(syncQueueSize_), image_mono_sub_, image_depth_sub_, info_sub_);
 				if(approxSyncMaxInterval > 0.0)
 					approxSync_->setMaxIntervalDuration(ros::Duration(approxSyncMaxInterval));
 				approxSync_->registerCallback(boost::bind(&RGBDOdometry::callback, this, boost::placeholders::_1, boost::placeholders::_2, boost::placeholders::_3));
 			}
 			else
 			{
-				exactSync_ = new message_filters::Synchronizer<MyExactSyncPolicy>(MyExactSyncPolicy(queueSize_), image_mono_sub_, image_depth_sub_, info_sub_);
+				exactSync_ = new message_filters::Synchronizer<MyExactSyncPolicy>(MyExactSyncPolicy(syncQueueSize_), image_mono_sub_, image_depth_sub_, info_sub_);
 				exactSync_->registerCallback(boost::bind(&RGBDOdometry::callback, this, boost::placeholders::_1, boost::placeholders::_2, boost::placeholders::_3));
 			}
 
+			subscribedTopic = rgb_nh.resolveName("image");
 			subscribedTopicsMsg = uFormat("\n%s subscribed to (%s sync%s):\n   %s \\\n   %s \\\n   %s",
 					getName().c_str(),
 					approxSync?"approx":"exact",
@@ -364,7 +401,7 @@ private:
 					image_depth_sub_.getTopic().c_str(),
 					info_sub_.getTopic().c_str());
 		}
-		this->startWarningThread(subscribedTopicsMsg, approxSync);
+		initDiagnosticMsg(subscribedTopicsMsg, approxSync, subscribedTopic);
 	}
 
 	virtual void updateParameters(ParametersMap & parameters)
@@ -546,7 +583,6 @@ private:
 			const sensor_msgs::ImageConstPtr& depth,
 			const sensor_msgs::CameraInfoConstPtr& cameraInfo)
 	{
-		callbackCalled();
 		if(!this->isPaused())
 		{
 			std::vector<cv_bridge::CvImageConstPtr> imageMsgs(1);
@@ -575,7 +611,6 @@ private:
 	void callbackRGBD(
 			const rtabmap_msgs::RGBDImageConstPtr& image)
 	{
-		callbackCalled();
 		if(!this->isPaused())
 		{
 			std::vector<cv_bridge::CvImageConstPtr> imageMsgs(1);
@@ -591,7 +626,6 @@ private:
 	void callbackRGBDX(
 			const rtabmap_msgs::RGBDImagesConstPtr& images)
 	{
-		callbackCalled();
 		if(!this->isPaused())
 		{
 			if(images->rgbd_images.empty())
@@ -616,7 +650,6 @@ private:
 			const rtabmap_msgs::RGBDImageConstPtr& image,
 			const rtabmap_msgs::RGBDImageConstPtr& image2)
 	{
-		callbackCalled();
 		if(!this->isPaused())
 		{
 			std::vector<cv_bridge::CvImageConstPtr> imageMsgs(2);
@@ -636,7 +669,6 @@ private:
 			const rtabmap_msgs::RGBDImageConstPtr& image2,
 			const rtabmap_msgs::RGBDImageConstPtr& image3)
 	{
-		callbackCalled();
 		if(!this->isPaused())
 		{
 			std::vector<cv_bridge::CvImageConstPtr> imageMsgs(3);
@@ -659,7 +691,6 @@ private:
 			const rtabmap_msgs::RGBDImageConstPtr& image3,
 			const rtabmap_msgs::RGBDImageConstPtr& image4)
 	{
-		callbackCalled();
 		if(!this->isPaused())
 		{
 			std::vector<cv_bridge::CvImageConstPtr> imageMsgs(4);
@@ -685,7 +716,6 @@ private:
 			const rtabmap_msgs::RGBDImageConstPtr& image4,
 			const rtabmap_msgs::RGBDImageConstPtr& image5)
 	{
-		callbackCalled();
 		if(!this->isPaused())
 		{
 			std::vector<cv_bridge::CvImageConstPtr> imageMsgs(5);
@@ -705,6 +735,35 @@ private:
 			this->commonCallback(imageMsgs, depthMsgs, infoMsgs);
 		}
 	}
+	void callbackRGBD6(
+			const rtabmap_msgs::RGBDImageConstPtr& image,
+			const rtabmap_msgs::RGBDImageConstPtr& image2,
+			const rtabmap_msgs::RGBDImageConstPtr& image3,
+			const rtabmap_msgs::RGBDImageConstPtr& image4,
+			const rtabmap_msgs::RGBDImageConstPtr& image5,
+			const rtabmap_msgs::RGBDImageConstPtr& image6)
+	{
+		if(!this->isPaused())
+		{
+			std::vector<cv_bridge::CvImageConstPtr> imageMsgs(6);
+			std::vector<cv_bridge::CvImageConstPtr> depthMsgs(6);
+			std::vector<sensor_msgs::CameraInfo> infoMsgs;
+			rtabmap_conversions::toCvShare(image, imageMsgs[0], depthMsgs[0]);
+			rtabmap_conversions::toCvShare(image2, imageMsgs[1], depthMsgs[1]);
+			rtabmap_conversions::toCvShare(image3, imageMsgs[2], depthMsgs[2]);
+			rtabmap_conversions::toCvShare(image4, imageMsgs[3], depthMsgs[3]);
+			rtabmap_conversions::toCvShare(image5, imageMsgs[4], depthMsgs[4]);
+			rtabmap_conversions::toCvShare(image6, imageMsgs[5], depthMsgs[5]);
+			infoMsgs.push_back(image->rgb_camera_info);
+			infoMsgs.push_back(image2->rgb_camera_info);
+			infoMsgs.push_back(image3->rgb_camera_info);
+			infoMsgs.push_back(image4->rgb_camera_info);
+			infoMsgs.push_back(image5->rgb_camera_info);
+			infoMsgs.push_back(image6->rgb_camera_info);
+
+			this->commonCallback(imageMsgs, depthMsgs, infoMsgs);
+		}
+	}
 
 protected:
 	virtual void flushCallbacks()
@@ -713,20 +772,20 @@ protected:
 		if(approxSync_)
 		{
 			delete approxSync_;
-			approxSync_ = new message_filters::Synchronizer<MyApproxSyncPolicy>(MyApproxSyncPolicy(queueSize_), image_mono_sub_, image_depth_sub_, info_sub_);
+			approxSync_ = new message_filters::Synchronizer<MyApproxSyncPolicy>(MyApproxSyncPolicy(syncQueueSize_), image_mono_sub_, image_depth_sub_, info_sub_);
 			approxSync_->registerCallback(boost::bind(&RGBDOdometry::callback, this, boost::placeholders::_1, boost::placeholders::_2, boost::placeholders::_3));
 		}
 		if(exactSync_)
 		{
 			delete exactSync_;
-			exactSync_ = new message_filters::Synchronizer<MyExactSyncPolicy>(MyExactSyncPolicy(queueSize_), image_mono_sub_, image_depth_sub_, info_sub_);
+			exactSync_ = new message_filters::Synchronizer<MyExactSyncPolicy>(MyExactSyncPolicy(syncQueueSize_), image_mono_sub_, image_depth_sub_, info_sub_);
 			exactSync_->registerCallback(boost::bind(&RGBDOdometry::callback, this, boost::placeholders::_1, boost::placeholders::_2, boost::placeholders::_3));
 		}
 		if(approxSync2_)
 		{
 			delete approxSync2_;
 			approxSync2_ = new message_filters::Synchronizer<MyApproxSync2Policy>(
-					MyApproxSync2Policy(queueSize_),
+					MyApproxSync2Policy(syncQueueSize_),
 					rgbd_image1_sub_,
 					rgbd_image2_sub_);
 			approxSync2_->registerCallback(boost::bind(&RGBDOdometry::callbackRGBD2, this, boost::placeholders::_1, boost::placeholders::_2));
@@ -735,7 +794,7 @@ protected:
 		{
 			delete exactSync2_;
 			exactSync2_ = new message_filters::Synchronizer<MyExactSync2Policy>(
-					MyExactSync2Policy(queueSize_),
+					MyExactSync2Policy(syncQueueSize_),
 					rgbd_image1_sub_,
 					rgbd_image2_sub_);
 			exactSync2_->registerCallback(boost::bind(&RGBDOdometry::callbackRGBD2, this, boost::placeholders::_1, boost::placeholders::_2));
@@ -744,7 +803,7 @@ protected:
 		{
 			delete approxSync3_;
 			approxSync3_ = new message_filters::Synchronizer<MyApproxSync3Policy>(
-					MyApproxSync3Policy(queueSize_),
+					MyApproxSync3Policy(syncQueueSize_),
 					rgbd_image1_sub_,
 					rgbd_image2_sub_,
 					rgbd_image3_sub_);
@@ -754,7 +813,7 @@ protected:
 		{
 			delete exactSync3_;
 			exactSync3_ = new message_filters::Synchronizer<MyExactSync3Policy>(
-					MyExactSync3Policy(queueSize_),
+					MyExactSync3Policy(syncQueueSize_),
 					rgbd_image1_sub_,
 					rgbd_image2_sub_,
 					rgbd_image3_sub_);
@@ -764,7 +823,7 @@ protected:
 		{
 			delete approxSync4_;
 			approxSync4_ = new message_filters::Synchronizer<MyApproxSync4Policy>(
-					MyApproxSync4Policy(queueSize_),
+					MyApproxSync4Policy(syncQueueSize_),
 					rgbd_image1_sub_,
 					rgbd_image2_sub_,
 					rgbd_image3_sub_,
@@ -775,7 +834,7 @@ protected:
 		{
 			delete exactSync4_;
 			exactSync4_ = new message_filters::Synchronizer<MyExactSync4Policy>(
-					MyExactSync4Policy(queueSize_),
+					MyExactSync4Policy(syncQueueSize_),
 					rgbd_image1_sub_,
 					rgbd_image2_sub_,
 					rgbd_image3_sub_,
@@ -786,7 +845,7 @@ protected:
 		{
 			delete approxSync5_;
 			approxSync5_ = new message_filters::Synchronizer<MyApproxSync5Policy>(
-					MyApproxSync5Policy(queueSize_),
+					MyApproxSync5Policy(syncQueueSize_),
 					rgbd_image1_sub_,
 					rgbd_image2_sub_,
 					rgbd_image3_sub_,
@@ -798,7 +857,7 @@ protected:
 		{
 			delete exactSync5_;
 			exactSync5_ = new message_filters::Synchronizer<MyExactSync5Policy>(
-					MyExactSync5Policy(queueSize_),
+					MyExactSync5Policy(syncQueueSize_),
 					rgbd_image1_sub_,
 					rgbd_image2_sub_,
 					rgbd_image3_sub_,
@@ -806,6 +865,32 @@ protected:
                                         rgbd_image5_sub_);
 			exactSync5_->registerCallback(boost::bind(&RGBDOdometry::callbackRGBD5, this, boost::placeholders::_1, boost::placeholders::_2, boost::placeholders::_3, boost::placeholders::_4, boost::placeholders::_5));
 		}
+		if(approxSync6_)
+		{
+			delete approxSync6_;
+			approxSync6_ = new message_filters::Synchronizer<MyApproxSync6Policy>(
+					MyApproxSync6Policy(syncQueueSize_),
+					rgbd_image1_sub_,
+					rgbd_image2_sub_,
+					rgbd_image3_sub_,
+					rgbd_image4_sub_,
+					rgbd_image5_sub_,
+					rgbd_image6_sub_);
+			approxSync6_->registerCallback(boost::bind(&RGBDOdometry::callbackRGBD6, this, boost::placeholders::_1, boost::placeholders::_2, boost::placeholders::_3, boost::placeholders::_4, boost::placeholders::_5, boost::placeholders::_6));
+		}
+		if(exactSync6_)
+		{
+			delete exactSync6_;
+			exactSync6_ = new message_filters::Synchronizer<MyExactSync6Policy>(
+					MyExactSync6Policy(syncQueueSize_),
+					rgbd_image1_sub_,
+					rgbd_image2_sub_,
+					rgbd_image3_sub_,
+					rgbd_image4_sub_,
+					rgbd_image5_sub_,
+					rgbd_image6_sub_);
+			exactSync6_->registerCallback(boost::bind(&RGBDOdometry::callbackRGBD6, this, boost::placeholders::_1, boost::placeholders::_2, boost::placeholders::_3, boost::placeholders::_4, boost::placeholders::_5, boost::placeholders::_6));
+		}
 	}
 
 private:
@@ -820,6 +905,7 @@ private:
 	message_filters::Subscriber<rtabmap_msgs::RGBDImage> rgbd_image3_sub_;
 	message_filters::Subscriber<rtabmap_msgs::RGBDImage> rgbd_image4_sub_;
 	message_filters::Subscriber<rtabmap_msgs::RGBDImage> rgbd_image5_sub_;
+	message_filters::Subscriber<rtabmap_msgs::RGBDImage> rgbd_image6_sub_;
 
 	typedef message_filters::sync_policies::ApproximateTime<sensor_msgs::Image, sensor_msgs::Image, sensor_msgs::CameraInfo> MyApproxSyncPolicy;
 	message_filters::Synchronizer<MyApproxSyncPolicy> * approxSync_;
@@ -841,7 +927,12 @@ private:
 	message_filters::Synchronizer<MyApproxSync5Policy> * approxSync5_;
 	typedef message_filters::sync_policies::ExactTime<rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage> MyExactSync5Policy;
 	message_filters::Synchronizer<MyExactSync5Policy> * exactSync5_;
-	int queueSize_;
+	typedef message_filters::sync_policies::ApproximateTime<rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage> MyApproxSync6Policy;
+	message_filters::Synchronizer<MyApproxSync6Policy> * approxSync6_;
+	typedef message_filters::sync_policies::ExactTime<rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage, rtabmap_msgs::RGBDImage> MyExactSync6Policy;
+	message_filters::Synchronizer<MyExactSync6Policy> * exactSync6_;
+	int topicQueueSize_;
+	int syncQueueSize_;
 	bool keepColor_;
 };
 
